@@ -1,7 +1,7 @@
 #
 #
 
-# Copyright (C) 2008 Google Inc.
+# Copyright (C) 2008, 2009, 2010 Google Inc.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -31,7 +31,9 @@ import tempfile
 import time
 import logging
 import pwd
-from cStringIO import StringIO
+import struct
+import fcntl
+import shutil
 
 from ganeti import utils
 from ganeti import constants
@@ -41,6 +43,88 @@ from ganeti import objects
 from ganeti import uidpool
 from ganeti import ssconf
 from ganeti.hypervisor import hv_base
+from ganeti import netutils
+from ganeti.utils import wrapper as utils_wrapper
+
+
+_KVM_NETWORK_SCRIPT = constants.SYSCONFDIR + "/ganeti/kvm-vif-bridge"
+
+# TUN/TAP driver constants, taken from <linux/if_tun.h>
+# They are architecture-independent and already hardcoded in qemu-kvm source,
+# so we can safely include them here.
+TUNSETIFF = 0x400454ca
+TUNGETIFF = 0x800454d2
+TUNGETFEATURES = 0x800454cf
+IFF_TAP = 0x0002
+IFF_NO_PI = 0x1000
+IFF_VNET_HDR = 0x4000
+
+
+def _ProbeTapVnetHdr(fd):
+  """Check whether to enable the IFF_VNET_HDR flag.
+
+  To do this, _all_ of the following conditions must be met:
+   1. TUNGETFEATURES ioctl() *must* be implemented
+   2. TUNGETFEATURES ioctl() result *must* contain the IFF_VNET_HDR flag
+   3. TUNGETIFF ioctl() *must* be implemented; reading the kernel code in
+      drivers/net/tun.c there is no way to test this until after the tap device
+      has been created using TUNSETIFF, and there is no way to change the
+      IFF_VNET_HDR flag after creating the interface, catch-22! However both
+      TUNGETIFF and TUNGETFEATURES were introduced in kernel version 2.6.27,
+      thus we can expect TUNGETIFF to be present if TUNGETFEATURES is.
+
+   @type fd: int
+   @param fd: the file descriptor of /dev/net/tun
+
+  """
+  req = struct.pack("I", 0)
+  try:
+    res = fcntl.ioctl(fd, TUNGETFEATURES, req)
+  except EnvironmentError:
+    logging.warning("TUNGETFEATURES ioctl() not implemented")
+    return False
+
+  tunflags = struct.unpack("I", res)[0]
+  if tunflags & IFF_VNET_HDR:
+    return True
+  else:
+    logging.warning("Host does not support IFF_VNET_HDR, not enabling")
+    return False
+
+
+def _OpenTap(vnet_hdr=True):
+  """Open a new tap device and return its file descriptor.
+
+  This is intended to be used by a qemu-type hypervisor together with the -net
+  tap,fd=<fd> command line parameter.
+
+  @type vnet_hdr: boolean
+  @param vnet_hdr: Enable the VNET Header
+  @return: (ifname, tapfd)
+  @rtype: tuple
+
+  """
+  try:
+    tapfd = os.open("/dev/net/tun", os.O_RDWR)
+  except EnvironmentError:
+    raise errors.HypervisorError("Failed to open /dev/net/tun")
+
+  flags = IFF_TAP | IFF_NO_PI
+
+  if vnet_hdr and _ProbeTapVnetHdr(tapfd):
+    flags |= IFF_VNET_HDR
+
+  # The struct ifreq ioctl request (see netdevice(7))
+  ifr = struct.pack("16sh", "", flags)
+
+  try:
+    res = fcntl.ioctl(tapfd, TUNSETIFF, ifr)
+  except EnvironmentError:
+    raise errors.HypervisorError("Failed to allocate a new TAP device")
+
+  # Get the interface name from the ioctl
+  ifname = struct.unpack("16sh", res)[0].strip("\x00")
+  return (ifname, tapfd)
 
 
 class KVMHypervisor(hv_base.BaseHypervisor):
@@ -52,6 +136,7 @@ class KVMHypervisor(hv_base.BaseHypervisor):
   _UIDS_DIR = _ROOT_DIR + "/uid" # contains instances reserved uids
   _CTRL_DIR = _ROOT_DIR + "/ctrl" # contains instances control sockets
   _CONF_DIR = _ROOT_DIR + "/conf" # contains instances startup data
+  _NICS_DIR = _ROOT_DIR + "/nic" # contains instances nic <-> tap associations
   # KVM instances with chroot enabled are started in empty chroot directories.
   _CHROOT_DIR = _ROOT_DIR + "/chroot" # for empty chroot directories
   # After an instance is stopped, its chroot directory is removed.
@@ -60,7 +145,7 @@ class KVMHypervisor(hv_base.BaseHypervisor):
   # To support forensics, the non-empty chroot directory is quarantined in
   # a separate directory, called 'chroot-quarantine'.
   _CHROOT_QUARANTINE_DIR = _ROOT_DIR + "/chroot-quarantine"
-  _DIRS = [_ROOT_DIR, _PIDS_DIR, _UIDS_DIR, _CTRL_DIR, _CONF_DIR,
+  _DIRS = [_ROOT_DIR, _PIDS_DIR, _UIDS_DIR, _CTRL_DIR, _CONF_DIR, _NICS_DIR,
            _CHROOT_DIR, _CHROOT_QUARANTINE_DIR]
 
   PARAMETERS = {
@@ -71,25 +156,31 @@ class KVMHypervisor(hv_base.BaseHypervisor):
     constants.HV_ACPI: hv_base.NO_CHECK,
     constants.HV_SERIAL_CONSOLE: hv_base.NO_CHECK,
     constants.HV_VNC_BIND_ADDRESS:
-      (False, lambda x: (utils.IsValidIP(x) or utils.IsNormAbsPath(x)),
+      (False, lambda x: (netutils.IP4Address.IsValid(x) or
+                         utils.IsNormAbsPath(x)),
        "the VNC bind address must be either a valid IP address or an absolute"
        " pathname", None, None),
     constants.HV_VNC_TLS: hv_base.NO_CHECK,
     constants.HV_VNC_X509: hv_base.OPT_DIR_CHECK,
     constants.HV_VNC_X509_VERIFY: hv_base.NO_CHECK,
     constants.HV_VNC_PASSWORD_FILE: hv_base.OPT_FILE_CHECK,
+    constants.HV_KVM_FLOPPY_IMAGE_PATH: hv_base.OPT_FILE_CHECK,
     constants.HV_CDROM_IMAGE_PATH: hv_base.OPT_FILE_CHECK,
+    constants.HV_KVM_CDROM2_IMAGE_PATH: hv_base.OPT_FILE_CHECK,
     constants.HV_BOOT_ORDER:
       hv_base.ParamInSet(True, constants.HT_KVM_VALID_BO_TYPES),
     constants.HV_NIC_TYPE:
       hv_base.ParamInSet(True, constants.HT_KVM_VALID_NIC_TYPES),
     constants.HV_DISK_TYPE:
       hv_base.ParamInSet(True, constants.HT_KVM_VALID_DISK_TYPES),
+    constants.HV_KVM_CDROM_DISK_TYPE:
+      hv_base.ParamInSet(False, constants.HT_KVM_VALID_DISK_TYPES),
     constants.HV_USB_MOUSE:
       hv_base.ParamInSet(False, constants.HT_KVM_VALID_MOUSE_TYPES),
     constants.HV_MIGRATION_PORT: hv_base.NET_PORT_CHECK,
     constants.HV_MIGRATION_BANDWIDTH: hv_base.NO_CHECK,
     constants.HV_MIGRATION_DOWNTIME: hv_base.NO_CHECK,
+    constants.HV_MIGRATION_MODE: hv_base.MIGRATION_MODE_CHECK,
     constants.HV_USE_LOCALTIME: hv_base.NO_CHECK,
     constants.HV_DISK_CACHE:
       hv_base.ParamInSet(True, constants.HT_VALID_CACHE_TYPES),
@@ -100,6 +191,7 @@ class KVMHypervisor(hv_base.BaseHypervisor):
       hv_base.ParamInSet(False, constants.HT_KVM_FLAG_VALUES),
     constants.HV_VHOST_NET: hv_base.NO_CHECK,
     constants.HV_KVM_USE_CHROOT: hv_base.NO_CHECK,
+    constants.HV_MEM_PATH: hv_base.OPT_DIR_CHECK,
     }
 
   _MIGRATION_STATUS_RE = re.compile('Migration\s+status:\s+(\w+)',
@@ -107,7 +199,7 @@ class KVMHypervisor(hv_base.BaseHypervisor):
   _MIGRATION_INFO_MAX_BAD_ANSWERS = 5
   _MIGRATION_INFO_RETRY_DELAY = 2
 
-  _KVM_NETWORK_SCRIPT = constants.SYSCONFDIR + "/ganeti/kvm-vif-bridge"
+  _VERSION_RE = re.compile(r"\b(\d+)\.(\d+)\.(\d+)\b")
 
   ANCILLARY_FILES = [
     _KVM_NETWORK_SCRIPT,
@@ -249,6 +341,21 @@ class KVMHypervisor(hv_base.BaseHypervisor):
     """
     return utils.PathJoin(cls._CHROOT_DIR, instance_name)
 
+  @classmethod
+  def _InstanceNICDir(cls, instance_name):
+    """Returns the name of the directory holding the tap device files for a
+    given instance.
+
+    """
+    return utils.PathJoin(cls._NICS_DIR, instance_name)
+
+  @classmethod
+  def _InstanceNICFile(cls, instance_name, seq):
+    """Returns the name of the file containing the tap device for a given NIC
+
+    """
+    return utils.PathJoin(cls._InstanceNICDir(instance_name), str(seq))
+
   @classmethod
   def _TryReadUidFile(cls, uid_file):
     """Try to read a uid file
@@ -278,6 +385,11 @@ class KVMHypervisor(hv_base.BaseHypervisor):
     utils.RemoveFile(uid_file)
     if uid is not None:
       uidpool.ReleaseUid(uid)
+    try:
+      shutil.rmtree(cls._InstanceNICDir(instance_name))
+    except OSError, err:
+      if err.errno != errno.ENOENT:
+        raise
     try:
       chroot_dir = cls._InstanceChrootDir(instance_name)
       utils.RemoveDir(chroot_dir)
@@ -297,10 +409,9 @@ class KVMHypervisor(hv_base.BaseHypervisor):
       else:
         raise
 
-  def _WriteNetScript(self, instance, seq, nic):
-    """Write a script to connect a net interface to the proper bridge.
-
-    This can be used by any qemu-type hypervisor.
+  @staticmethod
+  def _ConfigureNIC(instance, seq, nic, tap):
+    """Run the network configuration script for a specified NIC
 
     @param instance: instance we're acting on
     @type instance: instance object
@@ -308,68 +419,40 @@ class KVMHypervisor(hv_base.BaseHypervisor):
     @type seq: int
     @param nic: nic we're acting on
     @type nic: nic object
-    @return: netscript file name
-    @rtype: string
+    @param tap: the host's tap interface this NIC corresponds to
+    @type tap: str
 
     """
-    script = StringIO()
-    script.write("#!/bin/sh\n")
-    script.write("# this is autogenerated by Ganeti, please do not edit\n#\n")
-    script.write("PATH=$PATH:/sbin:/usr/sbin\n")
-    script.write("export INSTANCE=%s\n" % instance.name)
-    script.write("export MAC=%s\n" % nic.mac)
-    if nic.ip:
-      script.write("export IP=%s\n" % nic.ip)
-    script.write("export MODE=%s\n" % nic.nicparams[constants.NIC_MODE])
-    if nic.nicparams[constants.NIC_LINK]:
-      script.write("export LINK=%s\n" % nic.nicparams[constants.NIC_LINK])
-    if nic.nicparams[constants.NIC_MODE] == constants.NIC_MODE_BRIDGED:
-      script.write("export BRIDGE=%s\n" % nic.nicparams[constants.NIC_LINK])
-    script.write("export INTERFACE=$1\n")
+
     if instance.tags:
-      script.write("export TAGS=\"%s\"\n" % " ".join(instance.tags))
-    # TODO: make this configurable at ./configure time
-    script.write("if [ -x '%s' ]; then\n" % self._KVM_NETWORK_SCRIPT)
-    script.write("  # Execute the user-specific vif file\n")
-    script.write("  %s\n" % self._KVM_NETWORK_SCRIPT)
-    script.write("else\n")
-    script.write("  ifconfig $INTERFACE 0.0.0.0 up\n")
+      tags = " ".join(instance.tags)
+    else:
+      tags = ""
+
+    env = {
+      "PATH": "%s:/sbin:/usr/sbin" % os.environ["PATH"],
+      "INSTANCE": instance.name,
+      "MAC": nic.mac,
+      "MODE": nic.nicparams[constants.NIC_MODE],
+      "INTERFACE": tap,
+      "INTERFACE_INDEX": str(seq),
+      "TAGS": tags,
+    }
+
+    if nic.ip:
+      env["IP"] = nic.ip
+
+    if nic.nicparams[constants.NIC_LINK]:
+      env["LINK"] = nic.nicparams[constants.NIC_LINK]
+
     if nic.nicparams[constants.NIC_MODE] == constants.NIC_MODE_BRIDGED:
-      script.write("  # Connect the interface to the bridge\n")
-      script.write("  brctl addif $BRIDGE $INTERFACE\n")
-    elif nic.nicparams[constants.NIC_MODE] == constants.NIC_MODE_ROUTED:
-      if not nic.ip:
-        raise errors.HypervisorError("nic/%d is routed, but has no ip." % seq)
-      script.write("  # Route traffic targeted at the IP to the interface\n")
-      if nic.nicparams[constants.NIC_LINK]:
-        script.write("  while ip rule del dev $INTERFACE; do :; done\n")
-        script.write("  ip rule add dev $INTERFACE table $LINK\n")
-        script.write("  ip route replace $IP table $LINK proto static"
-                     " dev $INTERFACE\n")
-      else:
-        script.write("  ip route replace $IP proto static"
-                     " dev $INTERFACE\n")
-      interface_v4_conf = "/proc/sys/net/ipv4/conf/$INTERFACE"
-      interface_v6_conf = "/proc/sys/net/ipv6/conf/$INTERFACE"
-      script.write("  if [ -d %s ]; then\n" % interface_v4_conf)
-      script.write("    echo 1 > %s/proxy_arp\n" % interface_v4_conf)
-      script.write("    echo 1 > %s/forwarding\n" % interface_v4_conf)
-      script.write("  fi\n")
-      script.write("  if [ -d %s ]; then\n" % interface_v6_conf)
-      script.write("    echo 1 > %s/proxy_ndp\n" % interface_v6_conf)
-      script.write("    echo 1 > %s/forwarding\n" % interface_v6_conf)
-      script.write("  fi\n")
-    script.write("fi\n\n")
-    # As much as we'd like to put this in our _ROOT_DIR, that will happen to be
-    # mounted noexec sometimes, so we'll have to find another place.
-    (tmpfd, tmpfile_name) = tempfile.mkstemp()
-    tmpfile = os.fdopen(tmpfd, 'w')
-    try:
-      tmpfile.write(script.getvalue())
-    finally:
-      tmpfile.close()
-    os.chmod(tmpfile_name, 0755)
-    return tmpfile_name
+      env["BRIDGE"] = nic.nicparams[constants.NIC_LINK]
+
+    result = utils.RunCmd([constants.KVM_IFUP, tap], env=env)
+    if result.failed:
+      raise errors.HypervisorError("Failed to configure interface %s: %s."
+                                   " Network configuration script output: %s" %
+                                   (tap, result.fail_reason, result.output))
 
   def ListInstances(self):
     """Get the list of running instances.
@@ -438,6 +521,7 @@ class KVMHypervisor(hv_base.BaseHypervisor):
     hvp = instance.hvparams
     boot_disk = hvp[constants.HV_BOOT_ORDER] == constants.HT_BO_DISK
     boot_cdrom = hvp[constants.HV_BOOT_ORDER] == constants.HT_BO_CDROM
+    boot_floppy = hvp[constants.HV_BOOT_ORDER] == constants.HT_BO_FLOPPY
     boot_network = hvp[constants.HV_BOOT_ORDER] == constants.HT_BO_NETWORK
 
     if hvp[constants.HV_KVM_FLAG] == constants.HT_KVM_ENABLED:
@@ -479,22 +563,51 @@ class KVMHypervisor(hv_base.BaseHypervisor):
                                                 cache_val)
       kvm_cmd.extend(['-drive', drive_val])
 
+    #Now we can specify a different device type for CDROM devices.
+    cdrom_disk_type = hvp[constants.HV_KVM_CDROM_DISK_TYPE]
+    if not cdrom_disk_type:
+      cdrom_disk_type = disk_type
+
     iso_image = hvp[constants.HV_CDROM_IMAGE_PATH]
     if iso_image:
       options = ',format=raw,media=cdrom'
       if boot_cdrom:
         kvm_cmd.extend(['-boot', 'd'])
-        if disk_type != constants.HT_DISK_IDE:
+        if cdrom_disk_type != constants.HT_DISK_IDE:
+          options = '%s,boot=on,if=%s' % (options, constants.HT_DISK_IDE)
+        else:
           options = '%s,boot=on' % options
       else:
-        if disk_type == constants.HT_DISK_PARAVIRTUAL:
+        if cdrom_disk_type == constants.HT_DISK_PARAVIRTUAL:
           if_val = ',if=virtio'
         else:
-          if_val = ',if=%s' % disk_type
+          if_val = ',if=%s' % cdrom_disk_type
         options = '%s%s' % (options, if_val)
       drive_val = 'file=%s%s' % (iso_image, options)
       kvm_cmd.extend(['-drive', drive_val])
 
+    iso_image2 = hvp[constants.HV_KVM_CDROM2_IMAGE_PATH]
+    if iso_image2:
+      options = ',format=raw,media=cdrom'
+      if cdrom_disk_type == constants.HT_DISK_PARAVIRTUAL:
+        if_val = ',if=virtio'
+      else:
+        if_val = ',if=%s' % cdrom_disk_type
+      options = '%s%s' % (options, if_val)
+      drive_val = 'file=%s%s' % (iso_image2, options)
+      kvm_cmd.extend(['-drive', drive_val])
+
+    floppy_image = hvp[constants.HV_KVM_FLOPPY_IMAGE_PATH]
+    if floppy_image:
+      options = ',format=raw,media=disk'
+      if boot_floppy:
+        kvm_cmd.extend(['-boot', 'a'])
+        options = '%s,boot=on' % options
+      if_val = ',if=floppy'
+      options = '%s%s' % (options, if_val)
+      drive_val = 'file=%s%s' % (floppy_image, options)
+      kvm_cmd.extend(['-drive', drive_val])
+
     kernel_path = hvp[constants.HV_KERNEL_PATH]
     if kernel_path:
       kvm_cmd.extend(['-kernel', kernel_path])
@@ -507,17 +620,24 @@ class KVMHypervisor(hv_base.BaseHypervisor):
         root_append.append('console=ttyS0,38400')
       kvm_cmd.extend(['-append', ' '.join(root_append)])
 
+    mem_path = hvp[constants.HV_MEM_PATH]
+    if mem_path:
+      kvm_cmd.extend(["-mem-path", mem_path, "-mem-prealloc"])
+
     mouse_type = hvp[constants.HV_USB_MOUSE]
+    vnc_bind_address = hvp[constants.HV_VNC_BIND_ADDRESS]
+
     if mouse_type:
       kvm_cmd.extend(['-usb'])
       kvm_cmd.extend(['-usbdevice', mouse_type])
+    elif vnc_bind_address:
+      kvm_cmd.extend(['-usbdevice', constants.HT_MOUSE_TABLET])
 
-    vnc_bind_address = hvp[constants.HV_VNC_BIND_ADDRESS]
     if vnc_bind_address:
-      if utils.IsValidIP(vnc_bind_address):
+      if netutils.IP4Address.IsValid(vnc_bind_address):
         if instance.network_port > constants.VNC_BASE_PORT:
           display = instance.network_port - constants.VNC_BASE_PORT
-          if vnc_bind_address == '0.0.0.0':
+          if vnc_bind_address == constants.IP4_ADDRESS_ANY:
             vnc_arg = ':%d' % (display)
           else:
             vnc_arg = '%s:%d' % (vnc_bind_address, display)
@@ -547,10 +667,6 @@ class KVMHypervisor(hv_base.BaseHypervisor):
         vnc_arg = 'unix:%s/%s.vnc' % (vnc_bind_address, instance.name)
 
       kvm_cmd.extend(['-vnc', vnc_arg])
-
-      # Also add a tablet USB device to act as a mouse
-      # This solves various mouse alignment issues
-      kvm_cmd.extend(['-usbdevice', 'tablet'])
     else:
       kvm_cmd.extend(['-nographic'])
 
@@ -617,16 +733,23 @@ class KVMHypervisor(hv_base.BaseHypervisor):
     kvm_nics = [objects.NIC.FromDict(snic) for snic in serialized_nics]
     return (kvm_cmd, kvm_nics, hvparams)
 
-  def _RunKVMCmd(self, name, kvm_cmd):
+  def _RunKVMCmd(self, name, kvm_cmd, tap_fds=None):
     """Run the KVM cmd and check for errors
 
     @type name: string
     @param name: instance name
     @type kvm_cmd: list of strings
     @param kvm_cmd: runcmd input for kvm
+    @type tap_fds: list of int
+    @param tap_fds: fds of tap devices opened by Ganeti
 
     """
-    result = utils.RunCmd(kvm_cmd)
+    try:
+      result = utils.RunCmd(kvm_cmd, noclose_fds=tap_fds)
+    finally:
+      for fd in tap_fds:
+        utils_wrapper.CloseFdNoError(fd)
+
     if result.failed:
       raise errors.HypervisorError("Failed to start instance %s: %s (%s)" %
                                    (name, result.fail_reason, result.output))
@@ -657,6 +780,12 @@ class KVMHypervisor(hv_base.BaseHypervisor):
     kvm_cmd, kvm_nics, up_hvp = kvm_runtime
     up_hvp = objects.FillDict(conf_hvp, up_hvp)
 
+    kvm_version = self._GetKVMVersion()
+    if kvm_version:
+      _, v_major, v_min, _ = kvm_version
+    else:
+      raise errors.HypervisorError("Unable to get KVM version")
+
     # We know it's safe to run as a different user upon migration, so we'll use
     # the latest conf, from conf_hvp.
     security_model = conf_hvp[constants.HV_SECURITY_MODEL]
@@ -666,25 +795,45 @@ class KVMHypervisor(hv_base.BaseHypervisor):
     # We have reasons to believe changing something like the nic driver/type
     # upon migration won't exactly fly with the instance kernel, so for nic
     # related parameters we'll use up_hvp
+    tapfds = []
+    taps = []
     if not kvm_nics:
       kvm_cmd.extend(["-net", "none"])
     else:
+      vnet_hdr = False
       tap_extra = ""
       nic_type = up_hvp[constants.HV_NIC_TYPE]
       if nic_type == constants.HT_NIC_PARAVIRTUAL:
-        nic_model = "model=virtio"
+        # From version 0.12.0, kvm uses a new sintax for network configuration.
+        if (v_major, v_min) >= (0, 12):
+          nic_model = "virtio-net-pci"
+          vnet_hdr = True
+        else:
+          nic_model = "virtio"
+
         if up_hvp[constants.HV_VHOST_NET]:
-          tap_extra = ",vhost=on"
+          # vhost_net is only available from version 0.13.0 or newer
+          if (v_major, v_min) >= (0, 13):
+            tap_extra = ",vhost=on"
+          else:
+            raise errors.HypervisorError("vhost_net is configured"
+                                        " but it is not available")
       else:
-        nic_model = "model=%s" % nic_type
+        nic_model = nic_type
 
       for nic_seq, nic in enumerate(kvm_nics):
-        nic_val = "nic,vlan=%s,macaddr=%s,%s" % (nic_seq, nic.mac, nic_model)
-        script = self._WriteNetScript(instance, nic_seq, nic)
-        tap_val = "tap,vlan=%s,script=%s%s" % (nic_seq, script, tap_extra)
-        kvm_cmd.extend(["-net", nic_val])
-        kvm_cmd.extend(["-net", tap_val])
-        temp_files.append(script)
+        tapname, tapfd = _OpenTap(vnet_hdr)
+        tapfds.append(tapfd)
+        taps.append(tapname)
+        if (v_major, v_min) >= (0, 12):
+          nic_val = "%s,mac=%s,netdev=netdev%s" % (nic_model, nic.mac, nic_seq)
+          tap_val = "type=tap,id=netdev%s,fd=%d%s" % (nic_seq, tapfd, tap_extra)
+          kvm_cmd.extend(["-netdev", tap_val, "-device", nic_val])
+        else:
+          nic_val = "nic,vlan=%s,macaddr=%s,model=%s" % (nic_seq,
+                                                         nic.mac, nic_model)
+          tap_val = "tap,vlan=%s,fd=%d" % (nic_seq, tapfd)
+          kvm_cmd.extend(["-net", tap_val, "-net", nic_val])
 
     if incoming:
       target, port = incoming
@@ -706,6 +855,12 @@ class KVMHypervisor(hv_base.BaseHypervisor):
       utils.EnsureDirs([(self._InstanceChrootDir(name),
                          constants.SECURE_DIR_MODE)])
 
+    if not incoming:
+      # Configure the network now for starting instances, during
+      # FinalizeMigration for incoming instances
+      for nic_seq, nic in enumerate(kvm_nics):
+        self._ConfigureNIC(instance, nic_seq, nic, taps[nic_seq])
+
     if security_model == constants.HT_SM_POOL:
       ss = ssconf.SimpleStore()
       uid_pool = uidpool.ParseUidPool(ss.GetUidPool(), separator="\n")
@@ -714,7 +869,7 @@ class KVMHypervisor(hv_base.BaseHypervisor):
       try:
         username = pwd.getpwuid(uid.GetUid()).pw_name
         kvm_cmd.extend(["-runas", username])
-        self._RunKVMCmd(name, kvm_cmd)
+        self._RunKVMCmd(name, kvm_cmd, tapfds)
       except:
         uidpool.ReleaseUid(uid)
         raise
@@ -722,7 +877,13 @@ class KVMHypervisor(hv_base.BaseHypervisor):
         uid.Unlock()
         utils.WriteFile(self._InstanceUidFile(name), data=str(uid))
     else:
-      self._RunKVMCmd(name, kvm_cmd)
+      self._RunKVMCmd(name, kvm_cmd, tapfds)
+
+    utils.EnsureDirs([(self._InstanceNICDir(instance.name),
+                     constants.RUN_DIRS_MODE)])
+    for nic_seq, tap in enumerate(taps):
+      utils.WriteFile(self._InstanceNICFile(instance.name, nic_seq),
+                      data=tap)
 
     if vnc_pwd:
       change_cmd = 'change vnc password %s' % vnc_pwd
@@ -758,6 +919,23 @@ class KVMHypervisor(hv_base.BaseHypervisor):
 
     return result
 
+  @classmethod
+  def _GetKVMVersion(cls):
+    """Return the installed KVM version
+
+    @return: (version, v_maj, v_min, v_rev), or None
+
+    """
+    result = utils.RunCmd([constants.KVM_PATH, "--help"])
+    if result.failed:
+      return None
+    match = cls._VERSION_RE.search(result.output.splitlines()[0])
+    if not match:
+      return None
+
+    return (match.group(0), int(match.group(1)), int(match.group(2)),
+            int(match.group(3)))
+
   def StopInstance(self, instance, force=False, retry=False, name=None):
     """Stop an instance.
 
@@ -842,6 +1020,21 @@ class KVMHypervisor(hv_base.BaseHypervisor):
 
     """
     if success:
+      kvm_runtime = self._LoadKVMRuntime(instance, serialized_runtime=info)
+      kvm_nics = kvm_runtime[1]
+
+      for nic_seq, nic in enumerate(kvm_nics):
+        try:
+          tap = utils.ReadFile(self._InstanceNICFile(instance.name, nic_seq))
+        except EnvironmentError, err:
+          logging.warning("Failed to find host interface for %s NIC #%d: %s",
+                          instance.name, nic_seq, str(err))
+          continue
+        try:
+          self._ConfigureNIC(instance, nic_seq, nic, tap)
+        except errors.HypervisorError, err:
+          logging.warning(str(err))
+
       self._WriteKVMRuntime(instance.name, info)
     else:
       self.StopInstance(instance, force=True)
@@ -866,10 +1059,6 @@ class KVMHypervisor(hv_base.BaseHypervisor):
     if not alive:
       raise errors.HypervisorError("Instance not running, cannot migrate")
 
-    if not utils.TcpPing(target, port, live_port_needed=True):
-      raise errors.HypervisorError("Remote host %s not listening on port"
-                                   " %s, cannot migrate" % (target, port))
-
     if not live:
       self._CallMonitorCommand(instance_name, 'stop')
 
@@ -935,28 +1124,33 @@ class KVMHypervisor(hv_base.BaseHypervisor):
     return self.GetLinuxNodeInfo()
 
   @classmethod
-  def GetShellCommandForConsole(cls, instance, hvparams, beparams):
+  def GetInstanceConsole(cls, instance, hvparams, beparams):
     """Return a command for connecting to the console of an instance.
 
     """
     if hvparams[constants.HV_SERIAL_CONSOLE]:
-      shell_command = ("%s STDIO,%s UNIX-CONNECT:%s" %
-                       (constants.SOCAT_PATH, cls._SocatUnixConsoleParams(),
-                        utils.ShellQuote(cls._InstanceSerial(instance.name))))
-    else:
-      shell_command = "echo 'No serial shell for instance %s'" % instance.name
+      cmd = [constants.SOCAT_PATH,
+             "STDIO,%s" % cls._SocatUnixConsoleParams(),
+             "UNIX-CONNECT:%s" % cls._InstanceSerial(instance.name)]
+      return objects.InstanceConsole(instance=instance.name,
+                                     kind=constants.CONS_SSH,
+                                     host=instance.primary_node,
+                                     user=constants.GANETI_RUNAS,
+                                     command=cmd)
 
     vnc_bind_address = hvparams[constants.HV_VNC_BIND_ADDRESS]
-    if vnc_bind_address:
-      if instance.network_port > constants.VNC_BASE_PORT:
-        display = instance.network_port - constants.VNC_BASE_PORT
-        vnc_command = ("echo 'Instance has VNC listening on %s:%d"
-                       " (display: %d)'" % (vnc_bind_address,
-                                            instance.network_port,
-                                            display))
-        shell_command = "%s; %s" % (vnc_command, shell_command)
+    if vnc_bind_address and instance.network_port > constants.VNC_BASE_PORT:
+      display = instance.network_port - constants.VNC_BASE_PORT
+      return objects.InstanceConsole(instance=instance.name,
+                                     kind=constants.CONS_VNC,
+                                     host=vnc_bind_address,
+                                     port=instance.network_port,
+                                     display=display)
 
-    return shell_command
+    return objects.InstanceConsole(instance=instance.name,
+                                   kind=constants.CONS_MESSAGE,
+                                   message=("No serial shell for instance %s" %
+                                            instance.name))
 
   def Verify(self):
     """Verify the hypervisor.
@@ -969,7 +1163,6 @@ class KVMHypervisor(hv_base.BaseHypervisor):
     if not os.path.exists(constants.SOCAT_PATH):
       return "The socat binary ('%s') does not exist." % constants.SOCAT_PATH
 
-
   @classmethod
   def CheckParameterSyntax(cls, hvparams):
     """Check the given parameters for validity.
