@@ -1,7 +1,7 @@
 #
 #
 
-# Copyright (C) 2006, 2007, 2008 Google Inc.
+# Copyright (C) 2006, 2007, 2008, 2009, 2010 Google Inc.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -30,6 +30,8 @@ from ganeti import constants
 from ganeti import errors
 from ganeti import utils
 from ganeti.hypervisor import hv_base
+from ganeti import netutils
+from ganeti import objects
 
 
 class XenHypervisor(hv_base.BaseHypervisor):
@@ -293,12 +295,15 @@ class XenHypervisor(hv_base.BaseHypervisor):
     return result
 
   @classmethod
-  def GetShellCommandForConsole(cls, instance, hvparams, beparams):
+  def GetInstanceConsole(cls, instance, hvparams, beparams):
     """Return a command for connecting to the console of an instance.
 
     """
-    return "xm console %s" % instance.name
-
+    return objects.InstanceConsole(instance=instance.name,
+                                   kind=constants.CONS_SSH,
+                                   host=instance.primary_node,
+                                   user=constants.GANETI_RUNAS,
+                                   command=["xm", "console", instance.name])
 
   def Verify(self):
     """Verify the hypervisor.
@@ -311,7 +316,7 @@ class XenHypervisor(hv_base.BaseHypervisor):
       return "'xm info' failed: %s, %s" % (result.fail_reason, result.output)
 
   @staticmethod
-  def _GetConfigFileDiskData(block_devices):
+  def _GetConfigFileDiskData(block_devices, blockdev_prefix):
     """Get disk directive for xen config file.
 
     This method builds the xen config disk directive according to the
@@ -320,6 +325,8 @@ class XenHypervisor(hv_base.BaseHypervisor):
     @param block_devices: list of tuples (cfdev, rldev):
         - cfdev: dict containing ganeti config disk part
         - rldev: ganeti.bdev.BlockDev object
+    @param blockdev_prefix: a string containing blockdevice prefix,
+                            e.g. "sd" for /dev/sda
 
     @return: string containing disk directive for xen instance config file
 
@@ -332,9 +339,7 @@ class XenHypervisor(hv_base.BaseHypervisor):
     if len(block_devices) > 24:
       # 'z' - 'a' = 24
       raise errors.HypervisorError("Too many disks")
-    # FIXME: instead of this hardcoding here, each of PVM/HVM should
-    # directly export their info (currently HVM will just sed this info)
-    namespace = ["sd" + chr(i + ord('a')) for i in range(24)]
+    namespace = [blockdev_prefix + chr(i + ord('a')) for i in range(24)]
     for sd_name, (cfdev, dev_path) in zip(namespace, block_devices):
       if cfdev.mode == constants.DISK_RDWR:
         mode = "w"
@@ -409,7 +414,7 @@ class XenHypervisor(hv_base.BaseHypervisor):
 
     port = instance.hvparams[constants.HV_MIGRATION_PORT]
 
-    if not utils.TcpPing(target, port, live_port_needed=True):
+    if not netutils.TcpPing(target, port, live_port_needed=True):
       raise errors.HypervisorError("Remote host %s not listening on port"
                                    " %s, cannot migrate" % (target, port))
 
@@ -454,9 +459,12 @@ class XenPvmHypervisor(XenHypervisor):
     constants.HV_BOOTLOADER_ARGS: hv_base.NO_CHECK,
     constants.HV_KERNEL_PATH: hv_base.REQ_FILE_CHECK,
     constants.HV_INITRD_PATH: hv_base.OPT_FILE_CHECK,
-    constants.HV_ROOT_PATH: hv_base.REQUIRED_CHECK,
+    constants.HV_ROOT_PATH: hv_base.NO_CHECK,
     constants.HV_KERNEL_ARGS: hv_base.NO_CHECK,
     constants.HV_MIGRATION_PORT: hv_base.NET_PORT_CHECK,
+    constants.HV_MIGRATION_MODE: hv_base.MIGRATION_MODE_CHECK,
+    # TODO: Add a check for the blockdev prefix (matching [a-z:] or similar).
+    constants.HV_BLOCKDEV_PREFIX: hv_base.NO_CHECK,
     }
 
   @classmethod
@@ -507,12 +515,14 @@ class XenPvmHypervisor(XenHypervisor):
         nic_str += ", bridge=%s" % nic.nicparams[constants.NIC_LINK]
       vif_data.append("'%s'" % nic_str)
 
-    disk_data = cls._GetConfigFileDiskData(block_devices)
+    disk_data = cls._GetConfigFileDiskData(block_devices,
+                                           hvp[constants.HV_BLOCKDEV_PREFIX])
 
     config.write("vif = [%s]\n" % ",".join(vif_data))
     config.write("disk = [%s]\n" % ",".join(disk_data))
 
-    config.write("root = '%s'\n" % hvp[constants.HV_ROOT_PATH])
+    if hvp[constants.HV_ROOT_PATH]:
+      config.write("root = '%s'\n" % hvp[constants.HV_ROOT_PATH])
     config.write("on_poweroff = 'destroy'\n")
     config.write("on_reboot = 'restart'\n")
     config.write("on_crash = 'restart'\n")
@@ -549,13 +559,16 @@ class XenHvmHypervisor(XenHypervisor):
       hv_base.ParamInSet(True, constants.HT_HVM_VALID_NIC_TYPES),
     constants.HV_PAE: hv_base.NO_CHECK,
     constants.HV_VNC_BIND_ADDRESS:
-      (False, utils.IsValidIP,
+      (False, netutils.IP4Address.IsValid,
        "VNC bind address is not a valid IP address", None, None),
     constants.HV_KERNEL_PATH: hv_base.REQ_FILE_CHECK,
     constants.HV_DEVICE_MODEL: hv_base.REQ_FILE_CHECK,
     constants.HV_VNC_PASSWORD_FILE: hv_base.REQ_FILE_CHECK,
     constants.HV_MIGRATION_PORT: hv_base.NET_PORT_CHECK,
+    constants.HV_MIGRATION_MODE: hv_base.MIGRATION_MODE_CHECK,
     constants.HV_USE_LOCALTIME: hv_base.NO_CHECK,
+    # TODO: Add a check for the blockdev prefix (matching [a-z:] or similar).
+    constants.HV_BLOCKDEV_PREFIX: hv_base.NO_CHECK,
     }
 
   @classmethod
@@ -636,13 +649,10 @@ class XenHvmHypervisor(XenHypervisor):
       vif_data.append("'%s'" % nic_str)
 
     config.write("vif = [%s]\n" % ",".join(vif_data))
-    disk_data = cls._GetConfigFileDiskData(block_devices)
-    disk_type = hvp[constants.HV_DISK_TYPE]
-    if disk_type in (None, constants.HT_DISK_IOEMU):
-      replacement = ",ioemu:hd"
-    else:
-      replacement = ",hd"
-    disk_data = [line.replace(",sd", replacement) for line in disk_data]
+
+    disk_data = cls._GetConfigFileDiskData(block_devices,
+                                           hvp[constants.HV_BLOCKDEV_PREFIX])
+
     iso_path = hvp[constants.HV_CDROM_IMAGE_PATH]
     if iso_path:
       iso = "'file:%s,hdc:cdrom,r'" % iso_path
