@@ -19,20 +19,31 @@
 # 02110-1301, USA.
 
 
-"""Ganeti RAPI client."""
+"""Ganeti RAPI client.
+
+@attention: To use the RAPI client, the application B{must} call
+            C{pycurl.global_init} during initialization and
+            C{pycurl.global_cleanup} before exiting the process. This is very
+            important in multi-threaded programs. See curl_global_init(3) and
+            curl_global_cleanup(3) for details. The decorator L{UsesRapiClient}
+            can be used.
+
+"""
 
 # No Ganeti-specific modules should be imported. The RAPI client is supposed to
 # be standalone.
 
-import sys
-import httplib
-import urllib2
 import logging
 import simplejson
 import socket
 import urllib
-import OpenSSL
-import distutils.version
+import threading
+import pycurl
+
+try:
+  from cStringIO import StringIO
+except ImportError:
+  from StringIO import StringIO
 
 
 GANETI_RAPI_PORT = 5080
@@ -60,6 +71,7 @@ NODE_ROLE_REGULAR = "regular"
 # Internal constants
 _REQ_DATA_VERSION_FIELD = "__version__"
 _INST_CREATE_REQV1 = "instance-create-reqv1"
+_INST_REINSTALL_REQV1 = "instance-reinstall-reqv1"
 _INST_NIC_PARAMS = frozenset(["mac", "ip", "mode", "link", "bridge"])
 _INST_CREATE_V0_DISK_PARAMS = frozenset(["size"])
 _INST_CREATE_V0_PARAMS = frozenset([
@@ -68,6 +80,19 @@ _INST_CREATE_V0_PARAMS = frozenset([
   ])
 _INST_CREATE_V0_DPARAMS = frozenset(["beparams", "hvparams"])
 
+# Older pycURL versions don't have all error constants
+try:
+  _CURLE_SSL_CACERT = pycurl.E_SSL_CACERT
+  _CURLE_SSL_CACERT_BADFILE = pycurl.E_SSL_CACERT_BADFILE
+except AttributeError:
+  _CURLE_SSL_CACERT = 60
+  _CURLE_SSL_CACERT_BADFILE = 77
+
+_CURL_SSL_CERT_ERRORS = frozenset([
+  _CURLE_SSL_CACERT,
+  _CURLE_SSL_CACERT_BADFILE,
+  ])
+
 
 class Error(Exception):
   """Base error class for this module.
@@ -92,242 +117,126 @@ class GanetiApiError(Error):
     self.code = code
 
 
-def FormatX509Name(x509_name):
-  """Formats an X509 name.
-
-  @type x509_name: OpenSSL.crypto.X509Name
+def UsesRapiClient(fn):
+  """Decorator for code using RAPI client to initialize pycURL.
 
   """
-  try:
-    # Only supported in pyOpenSSL 0.7 and above
-    get_components_fn = x509_name.get_components
-  except AttributeError:
-    return repr(x509_name)
-  else:
-    return "".join("/%s=%s" % (name, value)
-                   for name, value in get_components_fn())
+  def wrapper(*args, **kwargs):
+    # curl_global_init(3) and curl_global_cleanup(3) must be called with only
+    # one thread running. This check is just a safety measure -- it doesn't
+    # cover all cases.
+    assert threading.activeCount() == 1, \
+           "Found active threads when initializing pycURL"
+
+    pycurl.global_init(pycurl.GLOBAL_ALL)
+    try:
+      return fn(*args, **kwargs)
+    finally:
+      pycurl.global_cleanup()
+
+  return wrapper
 
 
-class CertAuthorityVerify:
-  """Certificate verificator for SSL context.
+def GenericCurlConfig(verbose=False, use_signal=False,
+                      use_curl_cabundle=False, cafile=None, capath=None,
+                      proxy=None, verify_hostname=False,
+                      connect_timeout=None, timeout=None,
+                      _pycurl_version_fn=pycurl.version_info):
+  """Curl configuration function generator.
 
-  Configures SSL context to verify server's certificate.
+  @type verbose: bool
+  @param verbose: Whether to set cURL to verbose mode
+  @type use_signal: bool
+  @param use_signal: Whether to allow cURL to use signals
+  @type use_curl_cabundle: bool
+  @param use_curl_cabundle: Whether to use cURL's default CA bundle
+  @type cafile: string
+  @param cafile: In which file we can find the certificates
+  @type capath: string
+  @param capath: In which directory we can find the certificates
+  @type proxy: string
+  @param proxy: Proxy to use, None for default behaviour and empty string for
+                disabling proxies (see curl_easy_setopt(3))
+  @type verify_hostname: bool
+  @param verify_hostname: Whether to verify the remote peer certificate's
+                          commonName
+  @type connect_timeout: number
+  @param connect_timeout: Timeout for establishing connection in seconds
+  @type timeout: number
+  @param timeout: Timeout for complete transfer in seconds (see
+                  curl_easy_setopt(3)).
 
   """
-  _CAPATH_MINVERSION = "0.9"
-  _DEFVFYPATHS_MINVERSION = "0.9"
+  if use_curl_cabundle and (cafile or capath):
+    raise Error("Can not use default CA bundle when CA file or path is set")
 
-  _PYOPENSSL_VERSION = OpenSSL.__version__
-  _PARSED_PYOPENSSL_VERSION = distutils.version.LooseVersion(_PYOPENSSL_VERSION)
+  def _ConfigCurl(curl, logger):
+    """Configures a cURL object
 
-  _SUPPORT_CAPATH = (_PARSED_PYOPENSSL_VERSION >= _CAPATH_MINVERSION)
-  _SUPPORT_DEFVFYPATHS = (_PARSED_PYOPENSSL_VERSION >= _DEFVFYPATHS_MINVERSION)
-
-  def __init__(self, cafile=None, capath=None, use_default_verify_paths=False):
-    """Initializes this class.
-
-    @type cafile: string
-    @param cafile: In which file we can find the certificates
-    @type capath: string
-    @param capath: In which directory we can find the certificates
-    @type use_default_verify_paths: bool
-    @param use_default_verify_paths: Whether the platform provided CA
-                                     certificates are to be used for
-                                     verification purposes
+    @type curl: pycurl.Curl
+    @param curl: cURL object
 
     """
-    self._cafile = cafile
-    self._capath = capath
-    self._use_default_verify_paths = use_default_verify_paths
+    logger.debug("Using cURL version %s", pycurl.version)
 
-    if self._capath is not None and not self._SUPPORT_CAPATH:
-      raise Error(("PyOpenSSL %s has no support for a CA directory,"
-                   " version %s or above is required") %
-                  (self._PYOPENSSL_VERSION, self._CAPATH_MINVERSION))
+    # pycurl.version_info returns a tuple with information about the used
+    # version of libcurl. Item 5 is the SSL library linked to it.
+    # e.g.: (3, '7.18.0', 463360, 'x86_64-pc-linux-gnu', 1581, 'GnuTLS/2.0.4',
+    # 0, '1.2.3.3', ...)
+    sslver = _pycurl_version_fn()[5]
+    if not sslver:
+      raise Error("No SSL support in cURL")
 
-    if self._use_default_verify_paths and not self._SUPPORT_DEFVFYPATHS:
-      raise Error(("PyOpenSSL %s has no support for using default verification"
-                   " paths, version %s or above is required") %
-                  (self._PYOPENSSL_VERSION, self._DEFVFYPATHS_MINVERSION))
-
-  @staticmethod
-  def _VerifySslCertCb(logger, _, cert, errnum, errdepth, ok):
-    """Callback for SSL certificate verification.
-
-    @param logger: Logging object
-
-    """
-    if ok:
-      log_fn = logger.debug
+    lcsslver = sslver.lower()
+    if lcsslver.startswith("openssl/"):
+      pass
+    elif lcsslver.startswith("gnutls/"):
+      if capath:
+        raise Error("cURL linked against GnuTLS has no support for a"
+                    " CA path (%s)" % (pycurl.version, ))
     else:
-      log_fn = logger.error
+      raise NotImplementedError("cURL uses unsupported SSL version '%s'" %
+                                sslver)
 
-    log_fn("Verifying SSL certificate at depth %s, subject '%s', issuer '%s'",
-           errdepth, FormatX509Name(cert.get_subject()),
-           FormatX509Name(cert.get_issuer()))
+    curl.setopt(pycurl.VERBOSE, verbose)
+    curl.setopt(pycurl.NOSIGNAL, not use_signal)
 
-    if not ok:
-      try:
-        # Only supported in pyOpenSSL 0.7 and above
-        # pylint: disable-msg=E1101
-        fn = OpenSSL.crypto.X509_verify_cert_error_string
-      except AttributeError:
-        errmsg = ""
-      else:
-        errmsg = ":%s" % fn(errnum)
-
-      logger.error("verify error:num=%s%s", errnum, errmsg)
-
-    return ok
-
-  def __call__(self, ctx, logger):
-    """Configures an SSL context to verify certificates.
-
-    @type ctx: OpenSSL.SSL.Context
-    @param ctx: SSL context
-
-    """
-    if self._use_default_verify_paths:
-      ctx.set_default_verify_paths()
-
-    if self._cafile or self._capath:
-      if self._SUPPORT_CAPATH:
-        ctx.load_verify_locations(self._cafile, self._capath)
-      else:
-        ctx.load_verify_locations(self._cafile)
-
-    ctx.set_verify(OpenSSL.SSL.VERIFY_PEER,
-                   lambda conn, cert, errnum, errdepth, ok: \
-                     self._VerifySslCertCb(logger, conn, cert,
-                                           errnum, errdepth, ok))
-
-
-class _HTTPSConnectionOpenSSL(httplib.HTTPSConnection):
-  """HTTPS Connection handler that verifies the SSL certificate.
-
-  """
-  # Python before version 2.6 had its own httplib.FakeSocket wrapper for
-  # sockets
-  _SUPPORT_FAKESOCKET = (sys.hexversion < 0x2060000)
-
-  def __init__(self, *args, **kwargs):
-    """Initializes this class.
-
-    """
-    httplib.HTTPSConnection.__init__(self, *args, **kwargs)
-    self._logger = None
-    self._config_ssl_verification = None
-
-  def Setup(self, logger, config_ssl_verification):
-    """Sets the SSL verification config function.
-
-    @param logger: Logging object
-    @type config_ssl_verification: callable
-
-    """
-    assert self._logger is None
-    assert self._config_ssl_verification is None
-
-    self._logger = logger
-    self._config_ssl_verification = config_ssl_verification
-
-  def connect(self):
-    """Connect to the server specified when the object was created.
-
-    This ensures that SSL certificates are verified.
-
-    """
-    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-
-    ctx = OpenSSL.SSL.Context(OpenSSL.SSL.TLSv1_METHOD)
-    ctx.set_options(OpenSSL.SSL.OP_NO_SSLv2)
-
-    if self._config_ssl_verification:
-      self._config_ssl_verification(ctx, self._logger)
-
-    ssl = OpenSSL.SSL.Connection(ctx, sock)
-    ssl.connect((self.host, self.port))
-
-    if self._SUPPORT_FAKESOCKET:
-      self.sock = httplib.FakeSocket(sock, ssl)
+    # Whether to verify remote peer's CN
+    if verify_hostname:
+      # curl_easy_setopt(3): "When CURLOPT_SSL_VERIFYHOST is 2, that
+      # certificate must indicate that the server is the server to which you
+      # meant to connect, or the connection fails. [...] When the value is 1,
+      # the certificate must contain a Common Name field, but it doesn't matter
+      # what name it says. [...]"
+      curl.setopt(pycurl.SSL_VERIFYHOST, 2)
     else:
-      self.sock = _SslSocketWrapper(ssl)
+      curl.setopt(pycurl.SSL_VERIFYHOST, 0)
+
+    if cafile or capath or use_curl_cabundle:
+      # Require certificates to be checked
+      curl.setopt(pycurl.SSL_VERIFYPEER, True)
+      if cafile:
+        curl.setopt(pycurl.CAINFO, str(cafile))
+      if capath:
+        curl.setopt(pycurl.CAPATH, str(capath))
+      # Not changing anything for using default CA bundle
+    else:
+      # Disable SSL certificate verification
+      curl.setopt(pycurl.SSL_VERIFYPEER, False)
+
+    if proxy is not None:
+      curl.setopt(pycurl.PROXY, str(proxy))
+
+    # Timeouts
+    if connect_timeout is not None:
+      curl.setopt(pycurl.CONNECTTIMEOUT, connect_timeout)
+    if timeout is not None:
+      curl.setopt(pycurl.TIMEOUT, timeout)
+
+  return _ConfigCurl
 
 
-class _SslSocketWrapper(object):
-  def __init__(self, sock):
-    """Initializes this class.
-
-    """
-    self._sock = sock
-
-  def __getattr__(self, name):
-    """Forward everything to underlying socket.
-
-    """
-    return getattr(self._sock, name)
-
-  def makefile(self, mode, bufsize):
-    """Fake makefile method.
-
-    makefile() on normal file descriptors uses dup2(2), which doesn't work with
-    SSL sockets and therefore is not implemented by pyOpenSSL. This fake method
-    works with the httplib module, but might not work for other modules.
-
-    """
-    # pylint: disable-msg=W0212
-    return socket._fileobject(self._sock, mode, bufsize)
-
-
-class _HTTPSHandler(urllib2.HTTPSHandler):
-  def __init__(self, logger, config_ssl_verification):
-    """Initializes this class.
-
-    @param logger: Logging object
-    @type config_ssl_verification: callable
-    @param config_ssl_verification: Function to configure SSL context for
-                                    certificate verification
-
-    """
-    urllib2.HTTPSHandler.__init__(self)
-    self._logger = logger
-    self._config_ssl_verification = config_ssl_verification
-
-  def _CreateHttpsConnection(self, *args, **kwargs):
-    """Wrapper around L{_HTTPSConnectionOpenSSL} to add SSL verification.
-
-    This wrapper is necessary provide a compatible API to urllib2.
-
-    """
-    conn = _HTTPSConnectionOpenSSL(*args, **kwargs)
-    conn.Setup(self._logger, self._config_ssl_verification)
-    return conn
-
-  def https_open(self, req):
-    """Creates HTTPS connection.
-
-    Called by urllib2.
-
-    """
-    return self.do_open(self._CreateHttpsConnection, req)
-
-
-class _RapiRequest(urllib2.Request):
-  def __init__(self, method, url, headers, data):
-    """Initializes this class.
-
-    """
-    urllib2.Request.__init__(self, url, data=data, headers=headers)
-    self._method = method
-
-  def get_method(self):
-    """Returns the HTTP request method.
-
-    """
-    return self._method
-
-
-class GanetiRapiClient(object):
+class GanetiRapiClient(object): # pylint: disable-msg=R0904
   """Ganeti RAPI client.
 
   """
@@ -335,10 +244,9 @@ class GanetiRapiClient(object):
   _json_encoder = simplejson.JSONEncoder(sort_keys=True)
 
   def __init__(self, host, port=GANETI_RAPI_PORT,
-               username=None, password=None,
-               config_ssl_verification=None, ignore_proxy=False,
-               logger=logging):
-    """Constructor.
+               username=None, password=None, logger=logging,
+               curl_config_fn=None, curl_factory=None):
+    """Initializes this class.
 
     @type host: string
     @param host: the ganeti cluster master to interact with
@@ -348,41 +256,68 @@ class GanetiRapiClient(object):
     @param username: the username to connect with
     @type password: string
     @param password: the password to connect with
-    @type config_ssl_verification: callable
-    @param config_ssl_verification: Function to configure SSL context for
-                                    certificate verification
-    @type ignore_proxy: bool
-    @param ignore_proxy: Whether to ignore proxy settings
+    @type curl_config_fn: callable
+    @param curl_config_fn: Function to configure C{pycurl.Curl} object
     @param logger: Logging object
 
     """
-    self._host = host
-    self._port = port
+    self._username = username
+    self._password = password
     self._logger = logger
+    self._curl_config_fn = curl_config_fn
+    self._curl_factory = curl_factory
 
-    self._base_url = "https://%s:%s" % (host, port)
+    try:
+      socket.inet_pton(socket.AF_INET6, host)
+      address = "[%s]:%s" % (host, port)
+    except socket.error:
+      address = "%s:%s" % (host, port)
 
-    handlers = [_HTTPSHandler(self._logger, config_ssl_verification)]
+    self._base_url = "https://%s" % address
 
-    self._httpauthhandler = None
     if username is not None:
-      pwmgr = urllib2.HTTPPasswordMgrWithDefaultRealm()
-      pwmgr.add_password(None, self._base_url, username, password)
-      self._httpauthhandler = urllib2.HTTPBasicAuthHandler(pwmgr)
-      handlers.append(self._httpauthhandler)
+      if password is None:
+        raise Error("Password not specified")
     elif password:
       raise Error("Specified password without username")
 
-    if ignore_proxy:
-      handlers.append(urllib2.ProxyHandler({}))
+  def _CreateCurl(self):
+    """Creates a cURL object.
 
-    self._http = urllib2.build_opener(*handlers) # pylint: disable-msg=W0142
+    """
+    # Create pycURL object if no factory is provided
+    if self._curl_factory:
+      curl = self._curl_factory()
+    else:
+      curl = pycurl.Curl()
 
-    self._headers = {
-      "Accept": HTTP_APP_JSON,
-      "Content-type": HTTP_APP_JSON,
-      "User-Agent": self.USER_AGENT,
-      }
+    # Default cURL settings
+    curl.setopt(pycurl.VERBOSE, False)
+    curl.setopt(pycurl.FOLLOWLOCATION, False)
+    curl.setopt(pycurl.MAXREDIRS, 5)
+    curl.setopt(pycurl.NOSIGNAL, True)
+    curl.setopt(pycurl.USERAGENT, self.USER_AGENT)
+    curl.setopt(pycurl.SSL_VERIFYHOST, 0)
+    curl.setopt(pycurl.SSL_VERIFYPEER, False)
+    curl.setopt(pycurl.HTTPHEADER, [
+      "Accept: %s" % HTTP_APP_JSON,
+      "Content-type: %s" % HTTP_APP_JSON,
+      ])
+
+    assert ((self._username is None and self._password is None) ^
+            (self._username is not None and self._password is not None))
+
+    if self._username:
+      # Setup authentication
+      curl.setopt(pycurl.HTTPAUTH, pycurl.HTTPAUTH_BASIC)
+      curl.setopt(pycurl.USERPWD,
+                  str("%s:%s" % (self._username, self._password)))
+
+    # Call external configuration function
+    if self._curl_config_fn:
+      self._curl_config_fn(curl, self._logger)
+
+    return curl
 
   @staticmethod
   def _EncodeQuery(query):
@@ -436,10 +371,12 @@ class GanetiRapiClient(object):
     """
     assert path.startswith("/")
 
-    if content:
+    curl = self._CreateCurl()
+
+    if content is not None:
       encoded_content = self._json_encoder.encode(content)
     else:
-      encoded_content = None
+      encoded_content = ""
 
     # Build URL
     urlparts = [self._base_url, path]
@@ -449,35 +386,43 @@ class GanetiRapiClient(object):
 
     url = "".join(urlparts)
 
-    self._logger.debug("Sending request %s %s to %s:%s"
-                       " (headers=%r, content=%r)",
-                       method, url, self._host, self._port, self._headers,
-                       encoded_content)
+    self._logger.debug("Sending request %s %s (content=%r)",
+                       method, url, encoded_content)
 
-    req = _RapiRequest(method, url, self._headers, encoded_content)
+    # Buffer for response
+    encoded_resp_body = StringIO()
+
+    # Configure cURL
+    curl.setopt(pycurl.CUSTOMREQUEST, str(method))
+    curl.setopt(pycurl.URL, str(url))
+    curl.setopt(pycurl.POSTFIELDS, str(encoded_content))
+    curl.setopt(pycurl.WRITEFUNCTION, encoded_resp_body.write)
 
     try:
-      resp = self._http.open(req)
-      encoded_response_content = resp.read()
-      # Workaround for python 2.6: reset the retried count of the http handler,
-      # or it'll fail after ~5 requests made with the same client. Under python
-      # 2.4 and 2.5 this variable is not used.
-      if self._httpauthhandler is not None:
-        self._httpauthhandler.retried = 0
-    except (OpenSSL.SSL.Error, OpenSSL.crypto.Error), err:
-      raise CertificateError("SSL issue: %s (%r)" % (err, err))
-    except urllib2.HTTPError, err:
-      raise GanetiApiError(str(err), code=err.code)
-    except urllib2.URLError, err:
-      raise GanetiApiError(str(err))
+      # Send request and wait for response
+      try:
+        curl.perform()
+      except pycurl.error, err:
+        if err.args[0] in _CURL_SSL_CERT_ERRORS:
+          raise CertificateError("SSL certificate error %s" % err)
 
-    if encoded_response_content:
-      response_content = simplejson.loads(encoded_response_content)
+        raise GanetiApiError(str(err))
+    finally:
+      # Reset settings to not keep references to large objects in memory
+      # between requests
+      curl.setopt(pycurl.POSTFIELDS, "")
+      curl.setopt(pycurl.WRITEFUNCTION, lambda _: None)
+
+    # Get HTTP response code
+    http_code = curl.getinfo(pycurl.RESPONSE_CODE)
+
+    # Was anything written to the response buffer?
+    if encoded_resp_body.tell():
+      response_content = simplejson.loads(encoded_resp_body.getvalue())
     else:
       response_content = None
 
-    # TODO: Are there other status codes that are valid? (redirect?)
-    if resp.code != HTTP_OK:
+    if http_code != HTTP_OK:
       if isinstance(response_content, dict):
         msg = ("%s %s: %s" %
                (response_content["code"],
@@ -486,7 +431,7 @@ class GanetiRapiClient(object):
       else:
         msg = str(response_content)
 
-      raise GanetiApiError(msg, code=resp.code)
+      raise GanetiApiError(msg, code=http_code)
 
     return response_content
 
@@ -536,6 +481,30 @@ class GanetiRapiClient(object):
     return self._SendRequest(HTTP_GET, "/%s/info" % GANETI_RAPI_VERSION,
                              None, None)
 
+  def RedistributeConfig(self):
+    """Tells the cluster to redistribute its configuration files.
+
+    @return: job id
+
+    """
+    return self._SendRequest(HTTP_PUT,
+                             "/%s/redistribute-config" % GANETI_RAPI_VERSION,
+                             None, None)
+
+  def ModifyCluster(self, **kwargs):
+    """Modifies cluster parameters.
+
+    More details for parameters can be found in the RAPI documentation.
+
+    @rtype: int
+    @return: job id
+
+    """
+    body = kwargs
+
+    return self._SendRequest(HTTP_PUT,
+                             "/%s/modify" % GANETI_RAPI_VERSION, None, body)
+
   def GetClusterTags(self):
     """Gets the cluster tags.
 
@@ -781,6 +750,82 @@ class GanetiRapiClient(object):
                              ("/%s/instances/%s" %
                               (GANETI_RAPI_VERSION, instance)), query, None)
 
+  def ModifyInstance(self, instance, **kwargs):
+    """Modifies an instance.
+
+    More details for parameters can be found in the RAPI documentation.
+
+    @type instance: string
+    @param instance: Instance name
+    @rtype: int
+    @return: job id
+
+    """
+    body = kwargs
+
+    return self._SendRequest(HTTP_PUT,
+                             ("/%s/instances/%s/modify" %
+                              (GANETI_RAPI_VERSION, instance)), None, body)
+
+  def ActivateInstanceDisks(self, instance, ignore_size=None):
+    """Activates an instance's disks.
+
+    @type instance: string
+    @param instance: Instance name
+    @type ignore_size: bool
+    @param ignore_size: Whether to ignore recorded size
+    @return: job id
+
+    """
+    query = []
+    if ignore_size:
+      query.append(("ignore_size", 1))
+
+    return self._SendRequest(HTTP_PUT,
+                             ("/%s/instances/%s/activate-disks" %
+                              (GANETI_RAPI_VERSION, instance)), query, None)
+
+  def DeactivateInstanceDisks(self, instance):
+    """Deactivates an instance's disks.
+
+    @type instance: string
+    @param instance: Instance name
+    @return: job id
+
+    """
+    return self._SendRequest(HTTP_PUT,
+                             ("/%s/instances/%s/deactivate-disks" %
+                              (GANETI_RAPI_VERSION, instance)), None, None)
+
+  def GrowInstanceDisk(self, instance, disk, amount, wait_for_sync=None):
+    """Grows a disk of an instance.
+
+    More details for parameters can be found in the RAPI documentation.
+
+    @type instance: string
+    @param instance: Instance name
+    @type disk: integer
+    @param disk: Disk index
+    @type amount: integer
+    @param amount: Grow disk by this amount (MiB)
+    @type wait_for_sync: bool
+    @param wait_for_sync: Wait for disk to synchronize
+    @rtype: int
+    @return: job id
+
+    """
+    body = {
+      "amount": amount,
+      }
+
+    if wait_for_sync is not None:
+      body["wait_for_sync"] = wait_for_sync
+
+    return self._SendRequest(HTTP_POST,
+                             ("/%s/instances/%s/disk/%s/grow" %
+                              (GANETI_RAPI_VERSION, instance, disk)),
+                             None, body)
+
   def GetInstanceTags(self, instance):
     """Gets tags for an instance.
 
@@ -897,18 +942,39 @@ class GanetiRapiClient(object):
                              ("/%s/instances/%s/startup" %
                               (GANETI_RAPI_VERSION, instance)), query, None)
 
-  def ReinstallInstance(self, instance, os, no_startup=False):
+  def ReinstallInstance(self, instance, os=None, no_startup=False,
+                        osparams=None):
     """Reinstalls an instance.
 
     @type instance: str
-    @param instance: the instance to reinstall
-    @type os: str
-    @param os: the os to reinstall
+    @param instance: The instance to reinstall
+    @type os: str or None
+    @param os: The operating system to reinstall. If None, the instance's
+        current operating system will be installed again
     @type no_startup: bool
-    @param no_startup: whether to start the instance automatically
+    @param no_startup: Whether to start the instance automatically
 
     """
-    query = [("os", os)]
+    if _INST_REINSTALL_REQV1 in self.GetFeatures():
+      body = {
+        "start": not no_startup,
+        }
+      if os is not None:
+        body["os"] = os
+      if osparams is not None:
+        body["osparams"] = osparams
+      return self._SendRequest(HTTP_POST,
+                               ("/%s/instances/%s/reinstall" %
+                                (GANETI_RAPI_VERSION, instance)), None, body)
+
+    # Use old request format
+    if osparams:
+      raise GanetiApiError("Server does not support specifying OS parameters"
+                           " for instance reinstallation")
+
+    query = []
+    if os:
+      query.append(("os", os))
     if no_startup:
       query.append(("nostartup", 1))
     return self._SendRequest(HTTP_POST,
@@ -958,6 +1024,117 @@ class GanetiRapiClient(object):
                              ("/%s/instances/%s/replace-disks" %
                               (GANETI_RAPI_VERSION, instance)), query, None)
 
+  def PrepareExport(self, instance, mode):
+    """Prepares an instance for an export.
+
+    @type instance: string
+    @param instance: Instance name
+    @type mode: string
+    @param mode: Export mode
+    @rtype: string
+    @return: Job ID
+
+    """
+    query = [("mode", mode)]
+    return self._SendRequest(HTTP_PUT,
+                             ("/%s/instances/%s/prepare-export" %
+                              (GANETI_RAPI_VERSION, instance)), query, None)
+
+  def ExportInstance(self, instance, mode, destination, shutdown=None,
+                     remove_instance=None,
+                     x509_key_name=None, destination_x509_ca=None):
+    """Exports an instance.
+
+    @type instance: string
+    @param instance: Instance name
+    @type mode: string
+    @param mode: Export mode
+    @rtype: string
+    @return: Job ID
+
+    """
+    body = {
+      "destination": destination,
+      "mode": mode,
+      }
+
+    if shutdown is not None:
+      body["shutdown"] = shutdown
+
+    if remove_instance is not None:
+      body["remove_instance"] = remove_instance
+
+    if x509_key_name is not None:
+      body["x509_key_name"] = x509_key_name
+
+    if destination_x509_ca is not None:
+      body["destination_x509_ca"] = destination_x509_ca
+
+    return self._SendRequest(HTTP_PUT,
+                             ("/%s/instances/%s/export" %
+                              (GANETI_RAPI_VERSION, instance)), None, body)
+
+  def MigrateInstance(self, instance, mode=None, cleanup=None):
+    """Migrates an instance.
+
+    @type instance: string
+    @param instance: Instance name
+    @type mode: string
+    @param mode: Migration mode
+    @type cleanup: bool
+    @param cleanup: Whether to clean up a previously failed migration
+
+    """
+    body = {}
+
+    if mode is not None:
+      body["mode"] = mode
+
+    if cleanup is not None:
+      body["cleanup"] = cleanup
+
+    return self._SendRequest(HTTP_PUT,
+                             ("/%s/instances/%s/migrate" %
+                              (GANETI_RAPI_VERSION, instance)), None, body)
+
+  def RenameInstance(self, instance, new_name, ip_check=None, name_check=None):
+    """Changes the name of an instance.
+
+    @type instance: string
+    @param instance: Instance name
+    @type new_name: string
+    @param new_name: New instance name
+    @type ip_check: bool
+    @param ip_check: Whether to ensure instance's IP address is inactive
+    @type name_check: bool
+    @param name_check: Whether to ensure instance's name is resolvable
+
+    """
+    body = {
+      "new_name": new_name,
+      }
+
+    if ip_check is not None:
+      body["ip_check"] = ip_check
+
+    if name_check is not None:
+      body["name_check"] = name_check
+
+    return self._SendRequest(HTTP_PUT,
+                             ("/%s/instances/%s/rename" %
+                              (GANETI_RAPI_VERSION, instance)), None, body)
+
+  def GetInstanceConsole(self, instance):
+    """Request information for connecting to instance's console.
+
+    @type instance: string
+    @param instance: Instance name
+
+    """
+    return self._SendRequest(HTTP_GET,
+                             ("/%s/instances/%s/console" %
+                              (GANETI_RAPI_VERSION, instance)), None, None)
+
   def GetJobs(self):
     """Gets all jobs for the cluster.
 
@@ -1055,7 +1232,7 @@ class GanetiRapiClient(object):
                              None, None)
 
   def EvacuateNode(self, node, iallocator=None, remote_node=None,
-                   dry_run=False):
+                   dry_run=False, early_release=False):
     """Evacuates instances from a Ganeti node.
 
     @type node: str
@@ -1066,11 +1243,16 @@ class GanetiRapiClient(object):
     @param remote_node: node to evaucate to
     @type dry_run: bool
     @param dry_run: whether to perform a dry run
+    @type early_release: bool
+    @param early_release: whether to enable parallelization
 
-    @rtype: int
-    @return: job id
+    @rtype: list
+    @return: list of (job ID, instance name, new secondary node); if
+        dry_run was specified, then the actual move jobs were not
+        submitted and the job IDs will be C{None}
 
-    @raises GanetiApiError: if an iallocator and remote_node are both specified
+    @raises GanetiApiError: if an iallocator and remote_node are both
+        specified
 
     """
     if iallocator and remote_node:
@@ -1083,18 +1265,21 @@ class GanetiRapiClient(object):
       query.append(("remote_node", remote_node))
     if dry_run:
       query.append(("dry-run", 1))
+    if early_release:
+      query.append(("early_release", 1))
 
     return self._SendRequest(HTTP_POST,
                              ("/%s/nodes/%s/evacuate" %
                               (GANETI_RAPI_VERSION, node)), query, None)
 
-  def MigrateNode(self, node, live=True, dry_run=False):
+  def MigrateNode(self, node, mode=None, dry_run=False):
     """Migrates all primary instances from a node.
 
     @type node: str
     @param node: node to migrate
-    @type live: bool
-    @param live: whether to use live migration
+    @type mode: string
+    @param mode: if passed, it will overwrite the live migration type,
+        otherwise the hypervisor default will be used
     @type dry_run: bool
     @param dry_run: whether to perform a dry run
 
@@ -1103,8 +1288,8 @@ class GanetiRapiClient(object):
 
     """
     query = []
-    if live:
-      query.append(("live", 1))
+    if mode is not None:
+      query.append(("mode", mode))
     if dry_run:
       query.append(("dry-run", 1))
 
@@ -1280,3 +1465,149 @@ class GanetiRapiClient(object):
     return self._SendRequest(HTTP_DELETE,
                              ("/%s/nodes/%s/tags" %
                               (GANETI_RAPI_VERSION, node)), query, None)
+
+  def GetGroups(self, bulk=False):
+    """Gets all node groups in the cluster.
+
+    @type bulk: bool
+    @param bulk: whether to return all information about the groups
+
+    @rtype: list of dict or str
+    @return: if bulk is true, a list of dictionaries with info about all node
+        groups in the cluster, else a list of names of those node groups
+
+    """
+    query = []
+    if bulk:
+      query.append(("bulk", 1))
+
+    groups = self._SendRequest(HTTP_GET, "/%s/groups" % GANETI_RAPI_VERSION,
+                               query, None)
+    if bulk:
+      return groups
+    else:
+      return [g["name"] for g in groups]
+
+  def GetGroup(self, group):
+    """Gets information about a node group.
+
+    @type group: str
+    @param group: name of the node group whose info to return
+
+    @rtype: dict
+    @return: info about the node group
+
+    """
+    return self._SendRequest(HTTP_GET,
+                             "/%s/groups/%s" % (GANETI_RAPI_VERSION, group),
+                             None, None)
+
+  def CreateGroup(self, name, alloc_policy=None, dry_run=False):
+    """Creates a new node group.
+
+    @type name: str
+    @param name: the name of node group to create
+    @type alloc_policy: str
+    @param alloc_policy: the desired allocation policy for the group, if any
+    @type dry_run: bool
+    @param dry_run: whether to peform a dry run
+
+    @rtype: int
+    @return: job id
+
+    """
+    query = []
+    if dry_run:
+      query.append(("dry-run", 1))
+
+    body = {
+      "name": name,
+      "alloc_policy": alloc_policy
+      }
+
+    return self._SendRequest(HTTP_POST, "/%s/groups" % GANETI_RAPI_VERSION,
+                             query, body)
+
+  def ModifyGroup(self, group, **kwargs):
+    """Modifies a node group.
+
+    More details for parameters can be found in the RAPI documentation.
+
+    @type group: string
+    @param group: Node group name
+    @rtype: int
+    @return: job id
+
+    """
+    return self._SendRequest(HTTP_PUT,
+                             ("/%s/groups/%s/modify" %
+                              (GANETI_RAPI_VERSION, group)), None, kwargs)
+
+  def DeleteGroup(self, group, dry_run=False):
+    """Deletes a node group.
+
+    @type group: str
+    @param group: the node group to delete
+    @type dry_run: bool
+    @param dry_run: whether to peform a dry run
+
+    @rtype: int
+    @return: job id
+
+    """
+    query = []
+    if dry_run:
+      query.append(("dry-run", 1))
+
+    return self._SendRequest(HTTP_DELETE,
+                             ("/%s/groups/%s" %
+                              (GANETI_RAPI_VERSION, group)), query, None)
+
+  def RenameGroup(self, group, new_name):
+    """Changes the name of a node group.
+
+    @type group: string
+    @param group: Node group name
+    @type new_name: string
+    @param new_name: New node group name
+
+    @rtype: int
+    @return: job id
+
+    """
+    body = {
+      "new_name": new_name,
+      }
+
+    return self._SendRequest(HTTP_PUT,
+                             ("/%s/groups/%s/rename" %
+                              (GANETI_RAPI_VERSION, group)), None, body)
+
+
+  def AssignGroupNodes(self, group, nodes, force=False, dry_run=False):
+    """Assigns nodes to a group.
+
+    @type group: string
+    @param group: Node gropu name
+    @type nodes: list of strings
+    @param nodes: List of nodes to assign to the group
+
+    @rtype: int
+    @return: job id
+
+    """
+    query = []
+
+    if force:
+      query.append(("force", 1))
+
+    if dry_run:
+      query.append(("dry-run", 1))
+
+    body = {
+      "nodes": nodes,
+      }
+
+    return self._SendRequest(HTTP_PUT,
+                             ("/%s/groups/%s/assign-nodes" %
+                             (GANETI_RAPI_VERSION, group)), query, body)
