@@ -1,7 +1,7 @@
 #
 #
 
-# Copyright (C) 2006, 2007 Google Inc.
+# Copyright (C) 2006, 2007, 2010, 2011 Google Inc.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,10 +26,24 @@
 
 import os
 import logging
+import re
 
 from ganeti import utils
 from ganeti import errors
 from ganeti import constants
+from ganeti import netutils
+
+
+def FormatParamikoFingerprint(fingerprint):
+  """Format paramiko PKey fingerprint.
+
+  @type fingerprint: str
+  @param fingerprint: PKey fingerprint
+  @return: The string hex representation of the fingerprint
+
+  """
+  assert len(fingerprint) % 2 == 0
+  return ":".join(re.findall(r"..", fingerprint.lower()))
 
 
 def GetUserFiles(user, mkdir=False):
@@ -56,7 +70,7 @@ def GetUserFiles(user, mkdir=False):
   if mkdir:
     utils.EnsureDirs([(ssh_dir, constants.SECURE_DIR_MODE)])
   elif not os.path.isdir(ssh_dir):
-    raise errors.OpExecError("path ~%s/.ssh is not a directory" % user)
+    raise errors.OpExecError("Path %s is not a directory" % ssh_dir)
 
   return [utils.PathJoin(ssh_dir, base)
           for base in ["id_dsa", "id_dsa.pub", "authorized_keys"]]
@@ -66,8 +80,17 @@ class SshRunner:
   """Wrapper for SSH commands.
 
   """
-  def __init__(self, cluster_name):
+  def __init__(self, cluster_name, ipv6=False):
+    """Initializes this class.
+
+    @type cluster_name: str
+    @param cluster_name: name of the cluster
+    @type ipv6: bool
+    @param ipv6: If true, force ssh to use IPv6 addresses only
+
+    """
     self.cluster_name = cluster_name
+    self.ipv6 = ipv6
 
   def _BuildSshOptions(self, batch, ask_key, use_cluster_key,
                        strict_host_check, private_key=None, quiet=True):
@@ -83,7 +106,7 @@ class SshRunner:
     @param quiet: whether to enable -q to ssh
 
     @rtype: list
-    @return: the list of options ready to use in L{utils.RunCmd}
+    @return: the list of options ready to use in L{utils.process.RunCmd}
 
     """
     options = [
@@ -128,6 +151,9 @@ class SshRunner:
       else:
         options.append("-oStrictHostKeyChecking=no")
 
+    if self.ipv6:
+      options.append("-6")
+
     return options
 
   def BuildCmd(self, hostname, user, command, batch=True, ask_key=False,
@@ -168,8 +194,8 @@ class SshRunner:
 
     Args: see SshRunner.BuildCmd.
 
-    @rtype: L{utils.RunResult}
-    @return: the result as from L{utils.RunCmd()}
+    @rtype: L{utils.process.RunResult}
+    @return: the result as from L{utils.process.RunCmd()}
 
     """
     return utils.RunCmd(self.BuildCmd(*args, **kwargs))
@@ -195,6 +221,9 @@ class SshRunner:
     command = [constants.SCP, "-p"]
     command.extend(self._BuildSshOptions(True, False, True, True))
     command.append(filename)
+    if netutils.IP6Address.IsValid(node):
+      node = netutils.FormatAddress((node, None))
+
     command.append("%s:%s" % (node, filename))
 
     result = utils.RunCmd(command)
@@ -225,7 +254,7 @@ class SshRunner:
         - detail: string with details
 
     """
-    retval = self.Run(node, 'root', 'hostname --fqdn')
+    retval = self.Run(node, "root", "hostname --fqdn", quiet=False)
 
     if retval.failed:
       msg = "ssh problem"
@@ -243,7 +272,7 @@ class SshRunner:
       if node.startswith(remotehostname + "."):
         msg = "hostname not FQDN"
       else:
-        msg = "hostname mistmatch"
+        msg = "hostname mismatch"
       return False, ("%s: expected %s but got %s" %
                      (msg, node, remotehostname))
 
