@@ -1,7 +1,7 @@
 #
 #
 
-# Copyright (C) 2007 Google Inc.
+# Copyright (C) 2007, 2010, 2011 Google Inc.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,26 +24,24 @@
 """
 
 import tempfile
+import os.path
 
 from ganeti import constants
-from ganeti import bootstrap
 from ganeti import utils
 
 import qa_config
 import qa_utils
 import qa_error
 
-from qa_utils import AssertEqual, AssertNotEqual, StartSSH
+from qa_utils import AssertEqual, AssertCommand
 
 
 def _RemoveFileFromAllNodes(filename):
   """Removes a file from all nodes.
 
   """
-  for node in qa_config.get('nodes'):
-    cmd = ['rm', '-f', filename]
-    AssertEqual(StartSSH(node['primary'],
-                         utils.ShellQuoteArgs(cmd)).wait(), 0)
+  for node in qa_config.get("nodes"):
+    AssertCommand(["rm", "-f", filename], node=node)
 
 
 def _CheckFileOnAllNodes(filename, content):
@@ -51,15 +49,16 @@ def _CheckFileOnAllNodes(filename, content):
 
   """
   cmd = utils.ShellQuoteArgs(["cat", filename])
-  for node in qa_config.get('nodes'):
-    AssertEqual(qa_utils.GetCommandOutput(node['primary'], cmd),
-                content)
+  for node in qa_config.get("nodes"):
+    AssertEqual(qa_utils.GetCommandOutput(node["primary"], cmd), content)
 
 
 def TestClusterInit(rapi_user, rapi_secret):
   """gnt-cluster init"""
   master = qa_config.GetMasterNode()
 
+  rapi_dir = os.path.dirname(constants.RAPI_USERS_FILE)
+
   # First create the RAPI credentials
   fh = tempfile.NamedTemporaryFile()
   try:
@@ -68,19 +67,19 @@ def TestClusterInit(rapi_user, rapi_secret):
 
     tmpru = qa_utils.UploadFile(master["primary"], fh.name)
     try:
-      cmd = ["mv", tmpru, constants.RAPI_USERS_FILE]
-      AssertEqual(StartSSH(master["primary"],
-                           utils.ShellQuoteArgs(cmd)).wait(), 0)
+      AssertCommand(["mkdir", "-p", rapi_dir])
+      AssertCommand(["mv", tmpru, constants.RAPI_USERS_FILE])
     finally:
-      cmd = ["rm", "-f", tmpru]
-      AssertEqual(StartSSH(master["primary"],
-                           utils.ShellQuoteArgs(cmd)).wait(), 0)
+      AssertCommand(["rm", "-f", tmpru])
   finally:
     fh.close()
 
   # Initialize cluster
   cmd = ['gnt-cluster', 'init']
 
+  cmd.append("--primary-ip-version=%d" %
+             qa_config.get("primary_ip_version", 4))
+
   if master.get('secondary', None):
     cmd.append('--secondary-ip=%s' % master['secondary'])
 
@@ -95,14 +94,11 @@ def TestClusterInit(rapi_user, rapi_secret):
 
   cmd.append(qa_config.get('name'))
 
-  AssertEqual(StartSSH(master['primary'],
-                       utils.ShellQuoteArgs(cmd)).wait(), 0)
+  AssertCommand(cmd)
 
 
 def TestClusterRename():
   """gnt-cluster rename"""
-  master = qa_config.GetMasterNode()
-
   cmd = ['gnt-cluster', 'rename', '-f']
 
   original_name = qa_config.get('name')
@@ -111,58 +107,119 @@ def TestClusterRename():
     print qa_utils.FormatError('"rename" entry is missing')
     return
 
-  cmd_1 = cmd + [rename_target]
-  cmd_2 = cmd + [original_name]
-
   cmd_verify = ['gnt-cluster', 'verify']
 
-  AssertEqual(StartSSH(master['primary'],
-                       utils.ShellQuoteArgs(cmd_1)).wait(), 0)
+  for data in [
+    cmd + [rename_target],
+    cmd_verify,
+    cmd + [original_name],
+    cmd_verify,
+    ]:
+    AssertCommand(data)
 
-  AssertEqual(StartSSH(master['primary'],
-                       utils.ShellQuoteArgs(cmd_verify)).wait(), 0)
 
-  AssertEqual(StartSSH(master['primary'],
-                       utils.ShellQuoteArgs(cmd_2)).wait(), 0)
+def TestClusterOob():
+  """out-of-band framework"""
+  oob_path_exists = "/tmp/ganeti-qa-oob-does-exist-%s" % utils.NewUUID()
 
-  AssertEqual(StartSSH(master['primary'],
-                       utils.ShellQuoteArgs(cmd_verify)).wait(), 0)
+  AssertCommand(["gnt-cluster", "verify"])
+  AssertCommand(["gnt-cluster", "modify", "--node-parameters",
+                 "oob_program=/tmp/ganeti-qa-oob-does-not-exist-%s" %
+                 utils.NewUUID()])
+
+  AssertCommand(["gnt-cluster", "verify"], fail=True)
+
+  AssertCommand(["touch", oob_path_exists])
+  AssertCommand(["chmod", "0400", oob_path_exists])
+  AssertCommand(["gnt-cluster", "copyfile", oob_path_exists])
+
+  try:
+    AssertCommand(["gnt-cluster", "modify", "--node-parameters",
+                   "oob_program=%s" % oob_path_exists])
+
+    AssertCommand(["gnt-cluster", "verify"], fail=True)
+
+    AssertCommand(["chmod", "0500", oob_path_exists])
+    AssertCommand(["gnt-cluster", "copyfile", oob_path_exists])
+
+    AssertCommand(["gnt-cluster", "verify"])
+  finally:
+    AssertCommand(["gnt-cluster", "command", "rm", oob_path_exists])
+
+  AssertCommand(["gnt-cluster", "modify", "--node-parameters",
+                 "oob_program="])
 
 
 def TestClusterVerify():
   """gnt-cluster verify"""
-  master = qa_config.GetMasterNode()
+  AssertCommand(["gnt-cluster", "verify"])
+  AssertCommand(["gnt-cluster", "verify-disks"])
 
-  cmd = ['gnt-cluster', 'verify']
-  AssertEqual(StartSSH(master['primary'],
-                       utils.ShellQuoteArgs(cmd)).wait(), 0)
+
+def TestJobqueue():
+  """gnt-debug test-jobqueue"""
+  AssertCommand(["gnt-debug", "test-jobqueue"])
+
+
+def TestClusterReservedLvs():
+  """gnt-cluster reserved lvs"""
+  CVERIFY = ["gnt-cluster", "verify"]
+  for fail, cmd in [
+    (False, CVERIFY),
+    (False, ["gnt-cluster", "modify", "--reserved-lvs", ""]),
+    (False, ["lvcreate", "-L1G", "-nqa-test", "xenvg"]),
+    (True,  CVERIFY),
+    (False, ["gnt-cluster", "modify", "--reserved-lvs",
+             "xenvg/qa-test,.*/other-test"]),
+    (False, CVERIFY),
+    (False, ["gnt-cluster", "modify", "--reserved-lvs", ".*/qa-.*"]),
+    (False, CVERIFY),
+    (False, ["gnt-cluster", "modify", "--reserved-lvs", ""]),
+    (True,  CVERIFY),
+    (False, ["lvremove", "-f", "xenvg/qa-test"]),
+    (False, CVERIFY),
+    ]:
+    AssertCommand(cmd, fail=fail)
+
+
+def TestClusterModifyBe():
+  """gnt-cluster modify -B"""
+  for fail, cmd in [
+    # mem
+    (False, ["gnt-cluster", "modify", "-B", "memory=256"]),
+    (False, ["sh", "-c", "gnt-cluster info|grep '^ *memory: 256$'"]),
+    (True,  ["gnt-cluster", "modify", "-B", "memory=a"]),
+    (False, ["gnt-cluster", "modify", "-B", "memory=128"]),
+    (False, ["sh", "-c", "gnt-cluster info|grep '^ *memory: 128$'"]),
+    # vcpus
+    (False, ["gnt-cluster", "modify", "-B", "vcpus=4"]),
+    (False, ["sh", "-c", "gnt-cluster info|grep '^ *vcpus: 4$'"]),
+    (True,  ["gnt-cluster", "modify", "-B", "vcpus=a"]),
+    (False, ["gnt-cluster", "modify", "-B", "vcpus=1"]),
+    (False, ["sh", "-c", "gnt-cluster info|grep '^ *vcpus: 1$'"]),
+    # auto_balance
+    (False, ["gnt-cluster", "modify", "-B", "auto_balance=False"]),
+    (False, ["sh", "-c", "gnt-cluster info|grep '^ *auto_balance: False$'"]),
+    (True,  ["gnt-cluster", "modify", "-B", "auto_balance=1"]),
+    (False, ["gnt-cluster", "modify", "-B", "auto_balance=True"]),
+    (False, ["sh", "-c", "gnt-cluster info|grep '^ *auto_balance: True$'"]),
+    ]:
+    AssertCommand(cmd, fail=fail)
 
 
 def TestClusterInfo():
   """gnt-cluster info"""
-  master = qa_config.GetMasterNode()
-
-  cmd = ['gnt-cluster', 'info']
-  AssertEqual(StartSSH(master['primary'],
-                       utils.ShellQuoteArgs(cmd)).wait(), 0)
+  AssertCommand(["gnt-cluster", "info"])
 
 
 def TestClusterGetmaster():
   """gnt-cluster getmaster"""
-  master = qa_config.GetMasterNode()
-
-  cmd = ['gnt-cluster', 'getmaster']
-  AssertEqual(StartSSH(master['primary'],
-                       utils.ShellQuoteArgs(cmd)).wait(), 0)
+  AssertCommand(["gnt-cluster", "getmaster"])
 
 
 def TestClusterVersion():
   """gnt-cluster version"""
-  master = qa_config.GetMasterNode()
-
-  cmd = ['gnt-cluster', 'version']
-  AssertEqual(StartSSH(master['primary'],
-                       utils.ShellQuoteArgs(cmd)).wait(), 0)
+  AssertCommand(["gnt-cluster", "version"])
 
 
 def TestClusterRenewCrypto():
@@ -171,16 +228,18 @@ def TestClusterRenewCrypto():
 
   # Conflicting options
   cmd = ["gnt-cluster", "renew-crypto", "--force",
-         "--new-cluster-certificate", "--new-confd-hmac-key",
-         "--new-rapi-certificate", "--rapi-certificate=/dev/null"]
-  AssertNotEqual(StartSSH(master["primary"],
-                          utils.ShellQuoteArgs(cmd)).wait(), 0)
+         "--new-cluster-certificate", "--new-confd-hmac-key"]
+  conflicting = [
+    ["--new-rapi-certificate", "--rapi-certificate=/dev/null"],
+    ["--new-cluster-domain-secret", "--cluster-domain-secret=/dev/null"],
+    ]
+  for i in conflicting:
+    AssertCommand(cmd+i, fail=True)
 
   # Invalid RAPI certificate
   cmd = ["gnt-cluster", "renew-crypto", "--force",
          "--rapi-certificate=/dev/null"]
-  AssertNotEqual(StartSSH(master["primary"],
-                          utils.ShellQuoteArgs(cmd)).wait(), 0)
+  AssertCommand(cmd, fail=True)
 
   rapi_cert_backup = qa_utils.BackupFile(master["primary"],
                                          constants.RAPI_CERT_FILE)
@@ -191,35 +250,38 @@ def TestClusterRenewCrypto():
     # Ensure certificate doesn't cause "gnt-cluster verify" to complain
     validity = constants.SSL_CERT_EXPIRATION_WARN * 3
 
-    bootstrap.GenerateSelfSignedSslCert(fh.name, validity=validity)
+    utils.GenerateSelfSignedSslCert(fh.name, validity=validity)
 
     tmpcert = qa_utils.UploadFile(master["primary"], fh.name)
     try:
-      cmd = ["gnt-cluster", "renew-crypto", "--force",
-             "--rapi-certificate=%s" % tmpcert]
-      AssertEqual(StartSSH(master["primary"],
-                           utils.ShellQuoteArgs(cmd)).wait(), 0)
+      AssertCommand(["gnt-cluster", "renew-crypto", "--force",
+                     "--rapi-certificate=%s" % tmpcert])
     finally:
-      cmd = ["rm", "-f", tmpcert]
-      AssertEqual(StartSSH(master["primary"],
-                           utils.ShellQuoteArgs(cmd)).wait(), 0)
+      AssertCommand(["rm", "-f", tmpcert])
+
+    # Custom cluster domain secret
+    cds_fh = tempfile.NamedTemporaryFile()
+    cds_fh.write(utils.GenerateSecret())
+    cds_fh.write("\n")
+    cds_fh.flush()
+
+    tmpcds = qa_utils.UploadFile(master["primary"], cds_fh.name)
+    try:
+      AssertCommand(["gnt-cluster", "renew-crypto", "--force",
+                     "--cluster-domain-secret=%s" % tmpcds])
+    finally:
+      AssertCommand(["rm", "-f", tmpcds])
 
     # Normal case
-    cmd = ["gnt-cluster", "renew-crypto", "--force",
-           "--new-cluster-certificate", "--new-confd-hmac-key",
-           "--new-rapi-certificate"]
-    AssertEqual(StartSSH(master["primary"],
-                         utils.ShellQuoteArgs(cmd)).wait(), 0)
+    AssertCommand(["gnt-cluster", "renew-crypto", "--force",
+                   "--new-cluster-certificate", "--new-confd-hmac-key",
+                   "--new-rapi-certificate", "--new-cluster-domain-secret"])
 
     # Restore RAPI certificate
-    cmd = ["gnt-cluster", "renew-crypto", "--force",
-           "--rapi-certificate=%s" % rapi_cert_backup]
-    AssertEqual(StartSSH(master["primary"],
-                         utils.ShellQuoteArgs(cmd)).wait(), 0)
+    AssertCommand(["gnt-cluster", "renew-crypto", "--force",
+                   "--rapi-certificate=%s" % rapi_cert_backup])
   finally:
-    cmd = ["rm", "-f", rapi_cert_backup]
-    AssertEqual(StartSSH(master["primary"],
-                         utils.ShellQuoteArgs(cmd)).wait(), 0)
+    AssertCommand(["rm", "-f", rapi_cert_backup])
 
 
 def TestClusterBurnin():
@@ -267,30 +329,24 @@ def TestClusterBurnin():
       else:
         cmd.append('--reboot-types=%s' % ",".join(reboot_types))
       cmd += [inst['name'] for inst in instances]
-      AssertEqual(StartSSH(master['primary'],
-                           utils.ShellQuoteArgs(cmd)).wait(), 0)
+      AssertCommand(cmd)
     finally:
-      cmd = ['rm', '-f', script]
-      AssertEqual(StartSSH(master['primary'],
-                           utils.ShellQuoteArgs(cmd)).wait(), 0)
+      AssertCommand(["rm", "-f", script])
+
   finally:
     for inst in instances:
       qa_config.ReleaseInstance(inst)
 
 
 def TestClusterMasterFailover():
-  """gnt-cluster masterfailover"""
+  """gnt-cluster master-failover"""
   master = qa_config.GetMasterNode()
-
   failovermaster = qa_config.AcquireNode(exclude=master)
-  try:
-    cmd = ['gnt-cluster', 'masterfailover']
-    AssertEqual(StartSSH(failovermaster['primary'],
-                         utils.ShellQuoteArgs(cmd)).wait(), 0)
 
-    cmd = ['gnt-cluster', 'masterfailover']
-    AssertEqual(StartSSH(master['primary'],
-                         utils.ShellQuoteArgs(cmd)).wait(), 0)
+  cmd = ["gnt-cluster", "master-failover"]
+  try:
+    AssertCommand(cmd, node=failovermaster)
+    AssertCommand(cmd, node=master)
   finally:
     qa_config.ReleaseNode(failovermaster)
 
@@ -311,9 +367,7 @@ def TestClusterCopyfile():
   testname = qa_utils.UploadFile(master['primary'], f.name)
   try:
     # Copy file to all nodes
-    cmd = ['gnt-cluster', 'copyfile', testname]
-    AssertEqual(StartSSH(master['primary'],
-                         utils.ShellQuoteArgs(cmd)).wait(), 0)
+    AssertCommand(["gnt-cluster", "copyfile", testname])
     _CheckFileOnAllNodes(testname, uniqueid)
   finally:
     _RemoveFileFromAllNodes(testname)
@@ -321,8 +375,6 @@ def TestClusterCopyfile():
 
 def TestClusterCommand():
   """gnt-cluster command"""
-  master = qa_config.GetMasterNode()
-
   uniqueid = utils.NewUUID()
   rfile = "/tmp/gnt%s" % utils.NewUUID()
   rcmd = utils.ShellQuoteArgs(['echo', '-n', uniqueid])
@@ -330,7 +382,7 @@ def TestClusterCommand():
                               "%s >%s" % (rcmd, rfile)])
 
   try:
-    AssertEqual(StartSSH(master['primary'], cmd).wait(), 0)
+    AssertCommand(cmd)
     _CheckFileOnAllNodes(rfile, uniqueid)
   finally:
     _RemoveFileFromAllNodes(rfile)
@@ -338,8 +390,4 @@ def TestClusterCommand():
 
 def TestClusterDestroy():
   """gnt-cluster destroy"""
-  master = qa_config.GetMasterNode()
-
-  cmd = ['gnt-cluster', 'destroy', '--yes-do-it']
-  AssertEqual(StartSSH(master['primary'],
-                       utils.ShellQuoteArgs(cmd)).wait(), 0)
+  AssertCommand(["gnt-cluster", "destroy", "--yes-do-it"])
